# xmlrpc.rb
# Copyright (C) 2010-2014 Red Hat, Inc.
#
# Authors:
#   Akira TAGOH  <tagoh@redhat.com>
#
# This library is free software: you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'xmlrpc/client'

module Bugzilla

=begin rdoc

=== Bugzilla::XMLRPC

=end

  class XMLRPC

=begin rdoc

==== Bugzilla::XMLRPC#new(host, port = 443, path = '/xmlrpc.cgi', proxy_host = nil, proxy_port = nil)

=end

    def initialize(host, port = 443, path = '/xmlrpc.cgi', proxy_host = nil, proxy_port = nil, timeout = 60, http_basic_auth_user = nil, http_basic_auth_pass = nil)
      path ||= '/xmlrpc.cgi'
      use_ssl = port == 443 ? true : false
      @xmlrpc = ::XMLRPC::Client.new(host, path, port, proxy_host, proxy_port, http_basic_auth_user, http_basic_auth_pass, use_ssl, timeout)

      # workaround for https://bugs.ruby-lang.org/issues/8182
      @xmlrpc.http_header_extra = {'accept-encoding' => 'identity'}
    end # def initialize

=begin rdoc

==== Bugzilla::XMLRPC#call(cmd, params, user = nil, password = nil)

=end

    def call(cmd, params = {}, user = nil, password = nil)
      params = {} if params.nil?
      params['Bugzilla_login'] = user unless user.nil? || password.nil?
      params['Bugzilla_password'] = password unless user.nil? || password.nil?
      params['Bugzilla_token'] = @token unless @token.nil?
      @xmlrpc.http_header_extra['Authorization'] = "Bearer #{@api_key}" unless @api_key.nil?
      @xmlrpc.call(cmd, params)
    end # def call

=begin rdoc

==== Bugzilla::XMLRPC#cookie

=end

    def cookie
      @xmlrpc.cookie
    end # def cookie

=begin rdoc

==== Bugzilla::XMLRPC#cookie=(val)

=end

    def cookie=(val)
      @xmlrpc.cookie = val
    end # def cookie=

=begin rdoc

==== Bugzilla::XMLRPC#token

=end

    def token
      @token
    end # def token

=begin rdoc

==== Bugzilla::XMLRPC#token=(val)

=end

    def token=(val)
      @token = val
    end # def token=

=begin rdoc

==== Bugzilla::XMLRPC#api_key

=end

  def api_key
    @api_key
  end # def 

=begin rdoc

==== Bugzilla::XMLRPC#api_key=(val)

=end

  def api_key=(val)
    @api_key = val
  end # def 

  end # class XMLRPC

end # module Bugzilla
